package java.io;

import sun.misc.Unsafe;
import java.lang.reflect.Member;
import java.util.concurrent.ConcurrentHashMap;
import java.lang.ref.WeakReference;
import java.util.concurrent.ConcurrentMap;
import java.lang.ref.ReferenceQueue;
import java.security.NoSuchAlgorithmException;
import java.security.MessageDigest;
import java.util.Comparator;
import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.Collections;
import java.util.ArrayList;
import sun.misc.JavaSecurityAccess;
import java.security.AccessControlContext;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.UndeclaredThrowableException;
import sun.misc.SharedSecrets;
import java.util.HashSet;
import java.security.Permissions;
import java.lang.reflect.Proxy;
import java.lang.ref.SoftReference;
import java.lang.ref.Reference;
import sun.reflect.CallerSensitive;
import sun.reflect.misc.ReflectUtil;
import sun.reflect.Reflection;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.lang.reflect.Method;
import java.security.ProtectionDomain;
import java.lang.reflect.Constructor;
import sun.reflect.ReflectionFactory;

public class ObjectStreamClass implements Serializable
{
    public static final ObjectStreamField[] NO_FIELDS;
    private static final long serialVersionUID = -6120832682080437368L;
    private static final ObjectStreamField[] serialPersistentFields;
    private static boolean disableSerialConstructorChecks;
    private static final ReflectionFactory reflFactory;
    private Class<?> cl;
    private String name;
    private volatile Long suid;
    private boolean isProxy;
    private boolean isEnum;
    private boolean serializable;
    private boolean externalizable;
    private boolean hasWriteObjectData;
    private boolean hasBlockExternalData;
    private ClassNotFoundException resolveEx;
    private ObjectStreamClass.ExceptionInfo deserializeEx;
    private ObjectStreamClass.ExceptionInfo serializeEx;
    private ObjectStreamClass.ExceptionInfo defaultSerializeEx;
    private ObjectStreamField[] fields;
    private int primDataSize;
    private int numObjFields;
    private ObjectStreamClass.FieldReflector fieldRefl;
    private volatile ObjectStreamClass.ClassDataSlot[] dataLayout;
    private Constructor<?> cons;
    private ProtectionDomain[] domains;
    private Method writeObjectMethod;
    private Method readObjectMethod;
    private Method readObjectNoDataMethod;
    private Method writeReplaceMethod;
    private Method readResolveMethod;
    private ObjectStreamClass localDesc;
    private ObjectStreamClass superDesc;
    private boolean initialized;
    
    private static native void initNative();
    
    public static ObjectStreamClass lookup(Class<?> clazz) {
        return lookup(clazz, false);
    }
    
    public static ObjectStreamClass lookupAny(Class<?> clazz) {
        return lookup(clazz, true);
    }
    
    public String getName() {
        return this.name;
    }
    
    public long getSerialVersionUID() {
        if (this.suid == null) {
            this.suid = (Long)AccessController.doPrivileged(new PrivilegedAction<Long>() {
                @Override
                public Long run() {
                    return Long.valueOf(ObjectStreamClass.access$100(ObjectStreamClass.this.cl));
                }
            });
        }
        return (long)this.suid;
    }
    
    @CallerSensitive
    public Class<?> forClass() {
        if (this.cl == null) {
            return null;
        }
        this.requireInitialized();
        if (System.getSecurityManager() != null && ReflectUtil.needsPackageAccessCheck(Reflection.getCallerClass().getClassLoader(), this.cl.getClassLoader())) {
            ReflectUtil.checkPackageAccess(this.cl);
        }
        return this.cl;
    }
    
    public ObjectStreamField[] getFields() {
        return this.getFields(true);
    }
    
    public ObjectStreamField getField(String s) {
        return this.getField(s, null);
    }
    
    @Override
    public String toString() {
        return this.name + ": static final long serialVersionUID = " + this.getSerialVersionUID() + "L;";
    }
    
    static ObjectStreamClass lookup(Class<?> clazz, boolean b) {
        if (!b && !Serializable.class.isAssignableFrom(clazz)) {
            return null;
        }
        processQueue(ObjectStreamClass.Caches.localDescsQueue, ObjectStreamClass.Caches.localDescs);
        ObjectStreamClass.WeakClassKey weakClassKey = new ObjectStreamClass.WeakClassKey(clazz, ObjectStreamClass.Caches.localDescsQueue);
        Reference<Object> reference = (Reference)ObjectStreamClass.Caches.localDescs.get(weakClassKey);
        Object o = null;
        if (reference != null) {
            o = reference.get();
        }
        Object o2 = null;
        if (o == null) {
            ObjectStreamClass.EntryFuture referent = new ObjectStreamClass.EntryFuture();
            SoftReference o3 = new SoftReference(referent);
            do {
                if (reference != null) {
                    ObjectStreamClass.Caches.localDescs.remove(weakClassKey, reference);
                }
                reference = (Reference<?>)ObjectStreamClass.Caches.localDescs.putIfAbsent(weakClassKey, o3);
                if (reference != null) {
                    o = reference.get();
                }
                else {
                    continue;
                }
            } while (reference != null && o == null);
            if (o == null) {
                o2 = referent;
            }
        }
        if ((o instanceof ObjectStreamClass)) {
            return (ObjectStreamClass)o;
        }
        if ((o instanceof ObjectStreamClass.EntryFuture)) {
            o2 = (ObjectStreamClass.EntryFuture)o;
            if (o2.getOwner() == Thread.currentThread()) {
                o = null;
            }
            else {
                o = o2.get();
            }
        }
        if (o == null) {
            try {
                o = new ObjectStreamClass(clazz);
            }
            catch (Throwable t) {
                o = t;
            }
            if (o2.set(o)) {
                ObjectStreamClass.Caches.localDescs.put(weakClassKey, new SoftReference(o));
            }
            else {
                o = o2.get();
            }
        }
        if ((o instanceof ObjectStreamClass)) {
            return (ObjectStreamClass)o;
        }
        if ((o instanceof RuntimeException)) {
            throw (RuntimeException)o;
        }
        if ((o instanceof Error)) {
            throw (Error)o;
        }
        throw new InternalError("unexpected entry: " + o);
    }
    
    private ObjectStreamClass(final Class<?> clazz) {
        this.hasBlockExternalData = true;
        this.cl = clazz;
        this.name = clazz.getName();
        this.isProxy = Proxy.isProxyClass(clazz);
        this.isEnum = Enum.class.isAssignableFrom(clazz);
        this.serializable = Serializable.class.isAssignableFrom(clazz);
        this.externalizable = Externalizable.class.isAssignableFrom(clazz);
        Class<Object> superclass = clazz.getSuperclass();
        this.superDesc = ((superclass != null) ? lookup(superclass, false) : null);
        this.localDesc = this;
        if (this.serializable) {
            AccessController.doPrivileged(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    if (ObjectStreamClass.this.isEnum) {
                        ObjectStreamClass.this.suid = Long.valueOf(0L);
                        ObjectStreamClass.this.fields = ObjectStreamClass.NO_FIELDS;
                        return null;
                    }
                    if (clazz.isArray()) {
                        ObjectStreamClass.this.fields = ObjectStreamClass.NO_FIELDS;
                        return null;
                    }
                    ObjectStreamClass.this.suid = ObjectStreamClass.access$700(clazz);
                    try {
                        ObjectStreamClass.this.fields = ObjectStreamClass.access$800(clazz);
                        ObjectStreamClass.access$900(ObjectStreamClass.this);
                    }
                    catch (InvalidClassException ex) {
                        ObjectStreamClass.this.serializeEx = (ObjectStreamClass.this.deserializeEx = new ObjectStreamClass.ExceptionInfo(ex.classname, ex.getMessage()));
                        ObjectStreamClass.this.fields = ObjectStreamClass.NO_FIELDS;
                    }
                    if (ObjectStreamClass.this.externalizable) {
                        ObjectStreamClass.this.cons = ObjectStreamClass.access$1400(clazz);
                    }
                    else {
                        ObjectStreamClass.this.cons = ObjectStreamClass.access$1500(clazz);
                        ObjectStreamClass.this.writeObjectMethod = ObjectStreamClass.access$1700(clazz, "writeObject", new Class[] { ObjectOutputStream.class }, Void.TYPE);
                        ObjectStreamClass.this.readObjectMethod = ObjectStreamClass.access$1700(clazz, "readObject", new Class[] { ObjectInputStream.class }, Void.TYPE);
                        ObjectStreamClass.this.readObjectNoDataMethod = ObjectStreamClass.access$1700(clazz, "readObjectNoData", null, Void.TYPE);
                        ObjectStreamClass.this.hasWriteObjectData = (ObjectStreamClass.this.writeObjectMethod != null);
                    }
                    ObjectStreamClass.this.domains = ObjectStreamClass.access$2200(ObjectStreamClass.this, ObjectStreamClass.this.cons, clazz);
                    ObjectStreamClass.this.writeReplaceMethod = ObjectStreamClass.access$2400(clazz, "writeReplace", null, Object.class);
                    ObjectStreamClass.this.readResolveMethod = ObjectStreamClass.access$2400(clazz, "readResolve", null, Object.class);
                    return null;
                }
            });
        }
        else {
            this.suid = Long.valueOf(0L);
            this.fields = ObjectStreamClass.NO_FIELDS;
        }
        try {
            this.fieldRefl = getReflector(this.fields, this);
        }
        catch (InvalidClassException cause) {
            throw new InternalError(cause);
        }
        if (this.deserializeEx == null) {
            if (this.isEnum) {
                this.deserializeEx = new ObjectStreamClass.ExceptionInfo(this.name, "enum type");
            }
            else if (this.cons == null) {
                this.deserializeEx = new ObjectStreamClass.ExceptionInfo(this.name, "no valid constructor");
            }
        }
        for (int i = 0; i < this.fields.length; ++i) {
            if (this.fields[i].getField() == null) {
                this.defaultSerializeEx = new ObjectStreamClass.ExceptionInfo(this.name, "unmatched serializable field(s) declared");
            }
        }
        this.initialized = true;
    }
    
    ObjectStreamClass() {
        this.hasBlockExternalData = true;
    }
    
    private ProtectionDomain noPermissionsDomain() {
        Permissions permissions = new Permissions();
        permissions.setReadOnly();
        return new ProtectionDomain(null, permissions);
    }
    
    private ProtectionDomain[] getProtectionDomains(Constructor<?> constructor, Class<?> superclass) {
        ProtectionDomain[] array = null;
        if (constructor != null && superclass.getClassLoader() != null && System.getSecurityManager() != null) {
            Class<Object> declaringClass = constructor.getDeclaringClass();
            Object o = null;
            while (superclass != declaringClass) {
                ProtectionDomain protectionDomain = superclass.getProtectionDomain();
                if (protectionDomain != null) {
                    if (o == null) {
                        o = new HashSet();
                    }
                    o.add(protectionDomain);
                }
                superclass = superclass.getSuperclass();
                if (superclass == null) {
                    if (o == null) {
                        o = new HashSet();
                    }
                    else {
                        o.clear();
                    }
                    o.add(this.noPermissionsDomain());
                    break;
                }
                else {
                    continue;
                }
            }
            if (o != null) {
                array = (ProtectionDomain[])o.toArray(new ProtectionDomain[0]);
            }
        }
        return array;
    }
    
    void initProxy(Class<?> cl, ClassNotFoundException resolveEx, ObjectStreamClass superDesc) throws InvalidClassException {
        ObjectStreamClass lookup = null;
        if (cl != null) {
            lookup = lookup(cl, true);
            if (!lookup.isProxy) {
                throw new InvalidClassException("cannot bind proxy descriptor to a non-proxy class");
            }
        }
        this.cl = cl;
        this.resolveEx = resolveEx;
        this.superDesc = superDesc;
        this.isProxy = true;
        this.serializable = true;
        this.suid = Long.valueOf(0L);
        this.fields = ObjectStreamClass.NO_FIELDS;
        if (lookup != null) {
            this.localDesc = lookup;
            this.name = this.localDesc.name;
            this.externalizable = this.localDesc.externalizable;
            this.writeReplaceMethod = this.localDesc.writeReplaceMethod;
            this.readResolveMethod = this.localDesc.readResolveMethod;
            this.deserializeEx = this.localDesc.deserializeEx;
            this.domains = this.localDesc.domains;
            this.cons = this.localDesc.cons;
        }
        this.fieldRefl = getReflector(this.fields, this.localDesc);
        this.initialized = true;
    }
    
    void initNonProxy(ObjectStreamClass objectStreamClass, Class<?> cl, ClassNotFoundException resolveEx, ObjectStreamClass superDesc) throws InvalidClassException {
        long longValue = (long)Long.valueOf(objectStreamClass.getSerialVersionUID());
        ObjectStreamClass lookup = null;
        if (cl != null) {
            lookup = lookup(cl, true);
            if (lookup.isProxy) {
                throw new InvalidClassException("cannot bind non-proxy descriptor to a proxy class");
            }
            if (objectStreamClass.isEnum != lookup.isEnum) {
                throw new InvalidClassException(objectStreamClass.isEnum ? "cannot bind enum descriptor to a non-enum class" : "cannot bind non-enum descriptor to an enum class");
            }
            if (objectStreamClass.serializable == lookup.serializable && !cl.isArray() && longValue != lookup.getSerialVersionUID()) {
                throw new InvalidClassException(lookup.name, "local class incompatible: stream classdesc serialVersionUID = " + longValue + ", local class serialVersionUID = " + lookup.getSerialVersionUID());
            }
            if (!classNamesEqual(objectStreamClass.name, lookup.name)) {
                throw new InvalidClassException(lookup.name, "local class name incompatible with stream class name \"" + objectStreamClass.name + "\"");
            }
            if (!objectStreamClass.isEnum) {
                if (objectStreamClass.serializable == lookup.serializable && objectStreamClass.externalizable != lookup.externalizable) {
                    throw new InvalidClassException(lookup.name, "Serializable incompatible with Externalizable");
                }
                if (objectStreamClass.serializable != lookup.serializable || objectStreamClass.externalizable != lookup.externalizable || (!objectStreamClass.serializable && !objectStreamClass.externalizable)) {
                    this.deserializeEx = new ObjectStreamClass.ExceptionInfo(lookup.name, "class invalid for deserialization");
                }
            }
        }
        this.cl = cl;
        this.resolveEx = resolveEx;
        this.superDesc = superDesc;
        this.name = objectStreamClass.name;
        this.suid = Long.valueOf(longValue);
        this.isProxy = false;
        this.isEnum = objectStreamClass.isEnum;
        this.serializable = objectStreamClass.serializable;
        this.externalizable = objectStreamClass.externalizable;
        this.hasBlockExternalData = objectStreamClass.hasBlockExternalData;
        this.hasWriteObjectData = objectStreamClass.hasWriteObjectData;
        this.fields = objectStreamClass.fields;
        this.primDataSize = objectStreamClass.primDataSize;
        this.numObjFields = objectStreamClass.numObjFields;
        if (lookup != null) {
            this.localDesc = lookup;
            this.writeObjectMethod = this.localDesc.writeObjectMethod;
            this.readObjectMethod = this.localDesc.readObjectMethod;
            this.readObjectNoDataMethod = this.localDesc.readObjectNoDataMethod;
            this.writeReplaceMethod = this.localDesc.writeReplaceMethod;
            this.readResolveMethod = this.localDesc.readResolveMethod;
            if (this.deserializeEx == null) {
                this.deserializeEx = this.localDesc.deserializeEx;
            }
            this.domains = this.localDesc.domains;
            this.cons = this.localDesc.cons;
        }
        this.fieldRefl = getReflector(this.fields, this.localDesc);
        this.fields = this.fieldRefl.getFields();
        this.initialized = true;
    }
    
    void readNonProxy(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        this.name = objectInputStream.readUTF();
        this.suid = Long.valueOf(objectInputStream.readLong());
        this.isProxy = false;
        int byte1 = objectInputStream.readByte();
        this.hasWriteObjectData = ((byte1 & 0x1) != 0x0);
        this.hasBlockExternalData = ((byte1 & 0x8) != 0x0);
        this.externalizable = ((byte1 & 0x4) != 0x0);
        int n = (byte1 & 0x2) != 0x0;
        if (this.externalizable && n) {
            throw new InvalidClassException(this.name, "serializable and externalizable flags conflict");
        }
        this.serializable = (this.externalizable || n);
        this.isEnum = ((byte1 & 0x10) != 0x0);
        if (this.isEnum && (long)this.suid != 0L) {
            throw new InvalidClassException(this.name, "enum descriptor has non-zero serialVersionUID: " + this.suid);
        }
        int short1 = objectInputStream.readShort();
        if (this.isEnum && short1 != 0) {
            throw new InvalidClassException(this.name, "enum descriptor has non-zero field count: " + short1);
        }
        this.fields = ((short1 > 0) ? new ObjectStreamField[short1] : ObjectStreamClass.NO_FIELDS);
        for (int i = 0; i < short1; ++i) {
            int n2 = (char)objectInputStream.readByte();
            String utf = objectInputStream.readUTF();
            Object o = (n2 == 'L' || n2 == '[') ? objectInputStream.readTypeString() : new String(new char[] { n2 });
            try {
                this.fields[i] = new ObjectStreamField(utf, o, false);
            }
            catch (RuntimeException cause) {
                throw (IOException)new InvalidClassException(this.name, "invalid descriptor for field " + utf).initCause(cause);
            }
        }
        this.computeFieldOffsets();
        return;
    }
    
    void writeNonProxy(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.writeUTF(this.name);
        objectOutputStream.writeLong(this.getSerialVersionUID());
        int n = 0;
        if (this.externalizable) {
            n = (byte)(n | 0x4);
            if (objectOutputStream.getProtocolVersion() != 1) {
                n = (byte)(n | 0x8);
            }
        }
        else if (this.serializable) {
            n = (byte)(n | 0x2);
        }
        if (this.hasWriteObjectData) {
            n = (byte)(n | 0x1);
        }
        if (this.isEnum) {
            n = (byte)(n | 0x10);
        }
        objectOutputStream.writeByte(n);
        objectOutputStream.writeShort(this.fields.length);
        for (int i = 0; i < this.fields.length; ++i) {
            ObjectStreamField objectStreamField = this.fields[i];
            objectOutputStream.writeByte((int)objectStreamField.getTypeCode());
            objectOutputStream.writeUTF(objectStreamField.getName());
            if (!objectStreamField.isPrimitive()) {
                objectOutputStream.writeTypeString(objectStreamField.getTypeString());
            }
        }
    }
    
    ClassNotFoundException getResolveException() {
        return this.resolveEx;
    }
    
    private final void requireInitialized() {
        if (!this.initialized) {
            throw new InternalError("Unexpected call when not initialized");
        }
        return;
    }
    
    final void checkInitialized() throws InvalidClassException {
        if (!this.initialized) {
            throw new InvalidClassException("Class descriptor should be initialized");
        }
        return;
    }
    
    void checkDeserialize() throws InvalidClassException {
        this.requireInitialized();
        if (this.deserializeEx != null) {
            throw this.deserializeEx.newInvalidClassException();
        }
        return;
    }
    
    void checkSerialize() throws InvalidClassException {
        this.requireInitialized();
        if (this.serializeEx != null) {
            throw this.serializeEx.newInvalidClassException();
        }
        return;
    }
    
    void checkDefaultSerialize() throws InvalidClassException {
        this.requireInitialized();
        if (this.defaultSerializeEx != null) {
            throw this.defaultSerializeEx.newInvalidClassException();
        }
        return;
    }
    
    ObjectStreamClass getSuperDesc() {
        this.requireInitialized();
        return this.superDesc;
    }
    
    ObjectStreamClass getLocalDesc() {
        this.requireInitialized();
        return this.localDesc;
    }
    
    ObjectStreamField[] getFields(boolean b) {
        return b ? ((ObjectStreamField[])this.fields.clone()) : this.fields;
    }
    
    ObjectStreamField getField(String anObject, Class<?> clazz) {
        for (int i = 0; i < this.fields.length; ++i) {
            ObjectStreamField objectStreamField = this.fields[i];
            if (objectStreamField.getName().equals(anObject)) {
                if (clazz == null || (clazz == Object.class && !objectStreamField.isPrimitive())) {
                    return objectStreamField;
                }
                Class<Object> type = objectStreamField.getType();
                if (type != null && clazz.isAssignableFrom(type)) {
                    return objectStreamField;
                }
            }
        }
        return null;
    }
    
    boolean isProxy() {
        this.requireInitialized();
        return this.isProxy;
    }
    
    boolean isEnum() {
        this.requireInitialized();
        return this.isEnum;
    }
    
    boolean isExternalizable() {
        this.requireInitialized();
        return this.externalizable;
    }
    
    boolean isSerializable() {
        this.requireInitialized();
        return this.serializable;
    }
    
    boolean hasBlockExternalData() {
        this.requireInitialized();
        return this.hasBlockExternalData;
    }
    
    boolean hasWriteObjectData() {
        this.requireInitialized();
        return this.hasWriteObjectData;
    }
    
    boolean isInstantiable() {
        this.requireInitialized();
        return this.cons != null;
    }
    
    boolean hasWriteObjectMethod() {
        this.requireInitialized();
        return this.writeObjectMethod != null;
    }
    
    boolean hasReadObjectMethod() {
        this.requireInitialized();
        return this.readObjectMethod != null;
    }
    
    boolean hasReadObjectNoDataMethod() {
        this.requireInitialized();
        return this.readObjectNoDataMethod != null;
    }
    
    boolean hasWriteReplaceMethod() {
        this.requireInitialized();
        return this.writeReplaceMethod != null;
    }
    
    boolean hasReadResolveMethod() {
        this.requireInitialized();
        return this.readResolveMethod != null;
    }
    
    Object newInstance() throws InstantiationException, InvocationTargetException, UnsupportedOperationException {
        this.requireInitialized();
        if (this.cons != null) {
            try {
                if (this.domains == null || this.domains.length == 0) {
                    return this.cons.newInstance(new Object[0]);
                }
                JavaSecurityAccess javaSecurityAccess = SharedSecrets.getJavaSecurityAccess();
                PrivilegedAction<Object> privilegedAction = ()/*  */ -> {
                    try {
                        return this.cons.newInstance(new Object[0]);
                    }
                    catch (InstantiationException | InvocationTargetException | IllegalAccessException undeclaredThrowable) {
                        throw new UndeclaredThrowableException(undeclaredThrowable);
                    }
                };
                try {
                    return javaSecurityAccess.doIntersectionPrivilege(privilegedAction, AccessController.getContext(), new AccessControlContext(this.domains));
                }
                catch (UndeclaredThrowableException ex) {
                    Throwable cause = ex.getCause();
                    if ((cause instanceof InstantiationException)) {
                        throw (InstantiationException)cause;
                    }
                    if ((cause instanceof InvocationTargetException)) {
                        throw (InvocationTargetException)cause;
                    }
                    if ((cause instanceof IllegalAccessException)) {
                        throw (IllegalAccessException)cause;
                    }
                    throw ex;
                }
            }
            catch (IllegalAccessException cause2) {
                throw new InternalError(cause2);
            }
            catch (InstantiationError cause3) {
                InstantiationException o = new InstantiationException();
                o.initCause(cause3);
                throw o;
            }
        }
        throw new UnsupportedOperationException();
    }
    
    void invokeWriteObject(Object obj, ObjectOutputStream objectOutputStream) throws IOException, UnsupportedOperationException {
        this.requireInitialized();
        if (this.writeObjectMethod != null) {
            try {
                this.writeObjectMethod.invoke(obj, new Object[] { objectOutputStream });
            }
            catch (InvocationTargetException ex) {
                Throwable targetException = ex.getTargetException();
                if ((targetException instanceof IOException)) {
                    throw (IOException)targetException;
                }
                throwMiscException(targetException);
            }
            catch (IllegalAccessException cause) {
                throw new InternalError(cause);
            }
            return;
        }
        throw new UnsupportedOperationException();
    }
    
    void invokeReadObject(Object obj, ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException, UnsupportedOperationException {
        this.requireInitialized();
        if (this.readObjectMethod != null) {
            try {
                this.readObjectMethod.invoke(obj, new Object[] { objectInputStream });
            }
            catch (InvocationTargetException ex) {
                Throwable targetException = ex.getTargetException();
                if ((targetException instanceof ClassNotFoundException)) {
                    throw (ClassNotFoundException)targetException;
                }
                if ((targetException instanceof IOException)) {
                    throw (IOException)targetException;
                }
                throwMiscException(targetException);
            }
            catch (IllegalAccessException cause) {
                throw new InternalError(cause);
            }
            return;
        }
        throw new UnsupportedOperationException();
    }
    
    void invokeReadObjectNoData(Object obj) throws IOException, UnsupportedOperationException {
        this.requireInitialized();
        if (this.readObjectNoDataMethod != null) {
            try {
                this.readObjectNoDataMethod.invoke(obj, (Object[])null);
            }
            catch (InvocationTargetException ex) {
                Throwable targetException = ex.getTargetException();
                if ((targetException instanceof ObjectStreamException)) {
                    throw (ObjectStreamException)targetException;
                }
                throwMiscException(targetException);
            }
            catch (IllegalAccessException cause) {
                throw new InternalError(cause);
            }
            return;
        }
        throw new UnsupportedOperationException();
    }
    
    Object invokeWriteReplace(Object obj) throws IOException, UnsupportedOperationException {
        this.requireInitialized();
        if (this.writeReplaceMethod != null) {
            try {
                return this.writeReplaceMethod.invoke(obj, (Object[])null);
            }
            catch (InvocationTargetException ex) {
                Throwable targetException = ex.getTargetException();
                if ((targetException instanceof ObjectStreamException)) {
                    throw (ObjectStreamException)targetException;
                }
                throwMiscException(targetException);
                throw new InternalError(targetException);
            }
            catch (IllegalAccessException cause) {
                throw new InternalError(cause);
            }
        }
        else {
            throw new UnsupportedOperationException();
        }
    }
    
    Object invokeReadResolve(Object obj) throws IOException, UnsupportedOperationException {
        this.requireInitialized();
        if (this.readResolveMethod != null) {
            try {
                return this.readResolveMethod.invoke(obj, (Object[])null);
            }
            catch (InvocationTargetException ex) {
                Throwable targetException = ex.getTargetException();
                if ((targetException instanceof ObjectStreamException)) {
                    throw (ObjectStreamException)targetException;
                }
                throwMiscException(targetException);
                throw new InternalError(targetException);
            }
            catch (IllegalAccessException cause) {
                throw new InternalError(cause);
            }
        }
        else {
            throw new UnsupportedOperationException();
        }
    }
    
    ObjectStreamClass.ClassDataSlot[] getClassDataLayout() throws InvalidClassException {
        if (this.dataLayout == null) {
            this.dataLayout = this.getClassDataLayout0();
        }
        return this.dataLayout;
    }
    
    private ObjectStreamClass.ClassDataSlot[] getClassDataLayout0() throws InvalidClassException {
        ArrayList<ObjectStreamClass.ClassDataSlot> list = new ArrayList();
        Class<Object> clazz = this.cl;
        Class<Object> clazz2;
        for (clazz2 = this.cl; clazz2 != null && Serializable.class.isAssignableFrom(clazz2); clazz2 = clazz2.getSuperclass()) {}
        HashSet o = new HashSet(3);
        while (this != null) {
            if (o.contains(this.name)) {
                throw new InvalidClassException("Circular reference.");
            }
            o.add(this.name);
            String s = (this.cl != null) ? this.cl.getName() : this.name;
            Class<Object> clazz3 = null;
            while (clazz != clazz2) {
                if (s.equals(clazz.getName())) {
                    clazz3 = clazz;
                    break;
                }
                clazz = clazz.getSuperclass();
            }
            if (clazz3 != null) {
                for (; clazz != clazz3; clazz = clazz.getSuperclass()) {
                    list.add(new ObjectStreamClass.ClassDataSlot(lookup(clazz, true), false));
                }
                clazz = clazz3.getSuperclass();
            }
            list.add(new ObjectStreamClass.ClassDataSlot(this.getVariantFor(clazz3), true));
            this = this.superDesc;
        }
        for (; clazz != clazz2; clazz = clazz.getSuperclass()) {
            list.add(new ObjectStreamClass.ClassDataSlot(lookup(clazz, true), false));
        }
        Collections.reverse(list);
        return (ObjectStreamClass.ClassDataSlot[])list.toArray(new ObjectStreamClass.ClassDataSlot[list.size()]);
    }
    
    int getPrimDataSize() {
        return this.primDataSize;
    }
    
    int getNumObjFields() {
        return this.numObjFields;
    }
    
    void getPrimFieldValues(Object o, byte[] array) {
        this.fieldRefl.getPrimFieldValues(o, array);
    }
    
    void setPrimFieldValues(Object o, byte[] array) {
        this.fieldRefl.setPrimFieldValues(o, array);
    }
    
    void getObjFieldValues(Object o, Object[] array) {
        this.fieldRefl.getObjFieldValues(o, array);
    }
    
    void setObjFieldValues(Object o, Object[] array) {
        this.fieldRefl.setObjFieldValues(o, array);
    }
    
    private void computeFieldOffsets() throws InvalidClassException {
        this.primDataSize = 0;
        this.numObjFields = 0;
        int n = -1;
        for (int i = 0; i < this.fields.length; ++i) {
            ObjectStreamField objectStreamField = this.fields[i];
            switch (objectStreamField.getTypeCode()) {
                case 'B':
                case 'Z': {
                    objectStreamField.setOffset(this.primDataSize++);
                    break;
                }
                case 'C':
                case 'S': {
                    objectStreamField.setOffset(this.primDataSize);
                    this.primDataSize += 2;
                    break;
                }
                case 'F':
                case 'I': {
                    objectStreamField.setOffset(this.primDataSize);
                    this.primDataSize += 4;
                    break;
                }
                case 'D':
                case 'J': {
                    objectStreamField.setOffset(this.primDataSize);
                    this.primDataSize += 8;
                    break;
                }
                case 'L':
                case '[': {
                    objectStreamField.setOffset(this.numObjFields++);
                    if (n == -1) {
                        n = i;
                        break;
                    }
                    break;
                }
                default: {
                    throw new InternalError();
                }
            }
        }
        if (n != -1 && n + this.numObjFields != this.fields.length) {
            throw new InvalidClassException(this.name, "illegal field order");
        }
        return;
    }
    
    private ObjectStreamClass getVariantFor(Class<?> clazz) throws InvalidClassException {
        if (this.cl == clazz) {
            return this;
        }
        ObjectStreamClass objectStreamClass = new ObjectStreamClass();
        if (this.isProxy) {
            objectStreamClass.initProxy(clazz, null, this.superDesc);
            return objectStreamClass;
        }
        objectStreamClass.initNonProxy(this, clazz, null, this.superDesc);
        return objectStreamClass;
    }
    
    private static Constructor<?> getExternalizableConstructor(Class<?> clazz) {
        try {
            Constructor<Object> declaredConstructor = clazz.getDeclaredConstructor((Class[])null);
            declaredConstructor.setAccessible(true);
            return ((declaredConstructor.getModifiers() & 0x1) != 0x0) ? declaredConstructor : null;
        }
        catch (NoSuchMethodException ex) {
            return null;
        }
    }
    
    private static boolean superHasAccessibleConstructor(Class<?> clazz) {
        Class<Object> superclass = clazz.getSuperclass();
        assert !(!Serializable.class.isAssignableFrom(clazz));
        assert superclass != null;
        if (packageEquals(clazz, superclass)) {
            Constructor[] declaredConstructors2 = superclass.getDeclaredConstructors();
            int length2 = declaredConstructors2.length;
            int j = 0;
            while (j < length2) {
                if ((declaredConstructors2[j].getModifiers() & 0x2) == 0x0) {
                    return true;
                }
                ++j;
            }
            return false;
        }
        if ((superclass.getModifiers() & 0x5) == 0x0) {
            return false;
        }
        Constructor[] declaredConstructors = superclass.getDeclaredConstructors();
        int length = declaredConstructors.length;
        int i = 0;
        while (i < length) {
            if ((declaredConstructors[i].getModifiers() & 0x5) != 0x0) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    private static Constructor<?> getSerializableConstructor(Class<?> superclass) {
        while (Serializable.class.isAssignableFrom(superclass)) {
            if ((superclass = superclass.getSuperclass()) == null || (!ObjectStreamClass.disableSerialConstructorChecks && !superHasAccessibleConstructor(superclass))) {
                return null;
            }
            continue;
        }
        try {
            Constructor<Object> declaredConstructor = superclass.getDeclaredConstructor((Class[])null);
            int modifiers = declaredConstructor.getModifiers();
            if ((modifiers & 0x2) != 0x0 || ((modifiers & 0x5) == 0x0 && !packageEquals(superclass, superclass))) {
                return null;
            }
            Constructor<Object> constructorForSerialization = ObjectStreamClass.reflFactory.newConstructorForSerialization(superclass, declaredConstructor);
            constructorForSerialization.setAccessible(true);
            return constructorForSerialization;
        }
        catch (NoSuchMethodException ex) {
            return null;
        }
    }
    
    private static Method getInheritableMethod(Class<?> superclass, String name, Class<?>[] parameterTypes, Class<?> clazz) {
        Method declaredMethod = null;
        while (superclass != null) {
            try {
                declaredMethod = superclass.getDeclaredMethod(name, parameterTypes);
                break;
            }
            catch (NoSuchMethodException ex) {
                superclass = superclass.getSuperclass();
            }
        }
        if (declaredMethod == null || declaredMethod.getReturnType() != clazz) {
            return null;
        }
        declaredMethod.setAccessible(true);
        int modifiers = declaredMethod.getModifiers();
        if ((modifiers & 0x408) != 0x0) {
            return null;
        }
        if ((modifiers & 0x5) != 0x0) {
            return declaredMethod;
        }
        if ((modifiers & 0x2) != 0x0) {
            return (superclass == superclass) ? declaredMethod : null;
        }
        return packageEquals(superclass, superclass) ? declaredMethod : null;
    }
    
    private static Method getPrivateMethod(Class<?> clazz, String name, Class<?>[] parameterTypes, Class<?> clazz2) {
        try {
            Method declaredMethod = clazz.getDeclaredMethod(name, parameterTypes);
            declaredMethod.setAccessible(true);
            int modifiers = declaredMethod.getModifiers();
            return (declaredMethod.getReturnType() == clazz2 && (modifiers & 0x8) == 0x0 && (modifiers & 0x2) != 0x0) ? declaredMethod : null;
        }
        catch (NoSuchMethodException ex) {
            return null;
        }
    }
    
    private static boolean packageEquals(Class<?> clazz, Class<?> clazz2) {
        return clazz.getClassLoader() == clazz2.getClassLoader() && getPackageName(clazz).equals(getPackageName(clazz2));
    }
    
    private static String getPackageName(Class<?> clazz) {
        String s = clazz.getName();
        int lastIndex = s.lastIndexOf(91);
        if (lastIndex >= 0) {
            s = s.substring(lastIndex + 2);
        }
        int lastIndex2 = s.lastIndexOf(46);
        return (lastIndex2 >= 0) ? s.substring(0, lastIndex2) : "";
    }
    
    private static boolean classNamesEqual(String substring, String substring2) {
        substring = substring.substring(substring.lastIndexOf(46) + 1);
        substring2 = substring2.substring(substring2.lastIndexOf(46) + 1);
        return substring.equals(substring2);
    }
    
    private static String getClassSignature(Class<?> componentType) {
        StringBuilder sb = new StringBuilder();
        while (componentType.isArray()) {
            sb.append('[');
            componentType = componentType.getComponentType();
        }
        if (componentType.isPrimitive()) {
            if (componentType == Integer.TYPE) {
                sb.append('I');
            }
            else if (componentType == Byte.TYPE) {
                sb.append('B');
            }
            else if (componentType == Long.TYPE) {
                sb.append('J');
            }
            else if (componentType == Float.TYPE) {
                sb.append('F');
            }
            else if (componentType == Double.TYPE) {
                sb.append('D');
            }
            else if (componentType == Short.TYPE) {
                sb.append('S');
            }
            else if (componentType == Character.TYPE) {
                sb.append('C');
            }
            else if (componentType == Boolean.TYPE) {
                sb.append('Z');
            }
            else if (componentType == Void.TYPE) {
                sb.append('V');
            }
            else {
                throw new InternalError();
            }
        }
        else {
            sb.append(76 + componentType.getName().replace('.', '/') + 59);
        }
        return sb.toString();
    }
    
    private static String getMethodSignature(Class<?>[] array, Class<?> clazz) {
        StringBuilder sb = new StringBuilder();
        sb.append('(');
        for (int i = 0; i < array.length; ++i) {
            sb.append(getClassSignature(array[i]));
        }
        sb.append(')');
        sb.append(getClassSignature(clazz));
        return sb.toString();
    }
    
    private static void throwMiscException(Throwable cause) throws IOException {
        if ((cause instanceof RuntimeException)) {
            throw (RuntimeException)cause;
        }
        if ((cause instanceof Error)) {
            throw (Error)cause;
        }
        IOException ex = new IOException("unexpected exception type");
        ex.initCause(cause);
        throw ex;
    }
    
    private static ObjectStreamField[] getSerialFields(Class<?> cl) throws InvalidClassException {
        if (Serializable.class.isAssignableFrom(cl) && !Externalizable.class.isAssignableFrom(cl) && !Proxy.isProxyClass(cl) && !cl.isInterface()) {
            ObjectStreamField[] a;
            if ((a = getDeclaredSerialFields(cl)) == null) {
                a = getDefaultSerialFields(cl);
            }
            Arrays.sort(a);
            return a;
        }
        else {
            return ObjectStreamClass.NO_FIELDS;
        }
    }
    
    private static ObjectStreamField[] getDeclaredSerialFields(Class<?> clazz) throws InvalidClassException {
        ObjectStreamField[] array = null;
        try {
            Field declaredField = clazz.getDeclaredField("serialPersistentFields");
            int n = 26;
            if ((declaredField.getModifiers() & n) == n) {
                declaredField.setAccessible(true);
                array = (ObjectStreamField[])(ObjectStreamField[])declaredField.get(null);
            }
        }
        catch (Exception ex) {}
        if (array == null) {
            return null;
        }
        if (array.length == 0) {
            return ObjectStreamClass.NO_FIELDS;
        }
        ObjectStreamField[] array2 = new ObjectStreamField[array.length];
        HashSet o = new HashSet(array.length);
        int i = 0;
        while (i < array.length) {
            ObjectStreamField objectStreamField = array[i];
            String name = objectStreamField.getName();
            if (o.contains(name)) {
                throw new InvalidClassException("multiple serializable fields named " + name);
            }
            o.add(name);
            try {
                Field declaredField2 = clazz.getDeclaredField(name);
                if (declaredField2.getType() == objectStreamField.getType() && (declaredField2.getModifiers() & 0x8) == 0x0) {
                    array2[i] = new ObjectStreamField(declaredField2, objectStreamField.isUnshared(), true);
                }
            }
            catch (NoSuchFieldException ex2) {}
            if (array2[i] == null) {
                array2[i] = new ObjectStreamField(name, objectStreamField.getType(), objectStreamField.isUnshared());
            }
            ++i;
        }
        return array2;
    }
    
    private static ObjectStreamField[] getDefaultSerialFields(Class<?> clazz) {
        Field[] declaredFields = clazz.getDeclaredFields();
        ArrayList<ObjectStreamField> list = new ArrayList();
        int n = 136;
        for (int i = 0; i < declaredFields.length; ++i) {
            if ((declaredFields[i].getModifiers() & n) == 0x0) {
                list.add(new ObjectStreamField(declaredFields[i], false, true));
            }
        }
        int size = list.size();
        return (size == 0) ? ObjectStreamClass.NO_FIELDS : ((ObjectStreamField[])list.toArray(new ObjectStreamField[size]));
    }
    
    private static Long getDeclaredSUID(Class<?> clazz) {
        try {
            Field declaredField = clazz.getDeclaredField("serialVersionUID");
            int n = 24;
            if ((declaredField.getModifiers() & n) == n) {
                declaredField.setAccessible(true);
                return Long.valueOf(declaredField.getLong(null));
            }
        }
        catch (Exception ex) {}
        return null;
    }
    
    private static long computeDefaultSUID(Class<?> cl) {
        if (!Serializable.class.isAssignableFrom(cl) || Proxy.isProxyClass(cl)) {
            return 0L;
        }
        try {
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            DataOutputStream dataOutputStream = new DataOutputStream(byteArrayOutputStream);
            dataOutputStream.writeUTF(cl.getName());
            int n = cl.getModifiers() & 0x611;
            Method[] declaredMethods = cl.getDeclaredMethods();
            if ((n & 0x200) != 0x0) {
                n = ((declaredMethods.length > 0) ? (n | 0x400) : (n & 0xFFFFFBFF));
            }
            dataOutputStream.writeInt(n);
            if (!cl.isArray()) {
                Class[] interfaces = cl.getInterfaces();
                String[] a = new String[interfaces.length];
                for (int i = 0; i < interfaces.length; ++i) {
                    a[i] = interfaces[i].getName();
                }
                Arrays.sort(a);
                for (int j = 0; j < a.length; ++j) {
                    dataOutputStream.writeUTF(a[j]);
                }
            }
            Field[] declaredFields = cl.getDeclaredFields();
            ObjectStreamClass.MemberSignature[] a2 = new ObjectStreamClass.MemberSignature[declaredFields.length];
            for (int k = 0; k < declaredFields.length; ++k) {
                a2[k] = new ObjectStreamClass.MemberSignature(declaredFields[k]);
            }
            Arrays.sort(a2, new Comparator<ObjectStreamClass.MemberSignature>() {
                @Override
                public int compare(ObjectStreamClass.MemberSignature memberSignature, ObjectStreamClass.MemberSignature memberSignature2) {
                    return memberSignature.name.compareTo(memberSignature2.name);
                }
            });
            for (int l = 0; l < a2.length; ++l) {
                ObjectStreamClass.MemberSignature memberSignature = a2[l];
                int n2 = memberSignature.member.getModifiers() & 0xDF;
                if ((n2 & 0x2) == 0x0 || (n2 & 0x88) == 0x0) {
                    dataOutputStream.writeUTF(memberSignature.name);
                    dataOutputStream.writeInt(n2);
                    dataOutputStream.writeUTF(memberSignature.signature);
                }
            }
            if (hasStaticInitializer(cl)) {
                dataOutputStream.writeUTF("<clinit>");
                dataOutputStream.writeInt(8);
                dataOutputStream.writeUTF("()V");
            }
            Constructor[] declaredConstructors = cl.getDeclaredConstructors();
            ObjectStreamClass.MemberSignature[] a3 = new ObjectStreamClass.MemberSignature[declaredConstructors.length];
            for (int n3 = 0; n3 < declaredConstructors.length; ++n3) {
                a3[n3] = new ObjectStreamClass.MemberSignature(declaredConstructors[n3]);
            }
            Arrays.sort(a3, new Comparator<ObjectStreamClass.MemberSignature>() {
                @Override
                public int compare(ObjectStreamClass.MemberSignature memberSignature, ObjectStreamClass.MemberSignature memberSignature2) {
                    return memberSignature.signature.compareTo(memberSignature2.signature);
                }
            });
            for (int n4 = 0; n4 < a3.length; ++n4) {
                ObjectStreamClass.MemberSignature memberSignature2 = a3[n4];
                int n5 = memberSignature2.member.getModifiers() & 0xD3F;
                if ((n5 & 0x2) == 0x0) {
                    dataOutputStream.writeUTF("<init>");
                    dataOutputStream.writeInt(n5);
                    dataOutputStream.writeUTF(memberSignature2.signature.replace('/', '.'));
                }
            }
            ObjectStreamClass.MemberSignature[] a4 = new ObjectStreamClass.MemberSignature[declaredMethods.length];
            for (int n6 = 0; n6 < declaredMethods.length; ++n6) {
                a4[n6] = new ObjectStreamClass.MemberSignature(declaredMethods[n6]);
            }
            Arrays.sort(a4, new Comparator<ObjectStreamClass.MemberSignature>() {
                @Override
                public int compare(ObjectStreamClass.MemberSignature memberSignature, ObjectStreamClass.MemberSignature memberSignature2) {
                    int n = memberSignature.name.compareTo(memberSignature2.name);
                    if (n == 0) {
                        n = memberSignature.signature.compareTo(memberSignature2.signature);
                    }
                    return n;
                }
            });
            for (int n7 = 0; n7 < a4.length; ++n7) {
                ObjectStreamClass.MemberSignature memberSignature3 = a4[n7];
                int n8 = memberSignature3.member.getModifiers() & 0xD3F;
                if ((n8 & 0x2) == 0x0) {
                    dataOutputStream.writeUTF(memberSignature3.name);
                    dataOutputStream.writeInt(n8);
                    dataOutputStream.writeUTF(memberSignature3.signature.replace('/', '.'));
                }
            }
            dataOutputStream.flush();
            byte[] digest = MessageDigest.getInstance("SHA").digest(byteArrayOutputStream.toByteArray());
            long n9 = 0L;
            for (int n10 = Math.min(digest.length, 8) - 1; n10 >= 0; --n10) {
                n9 = (n9 << 8 | (long)(digest[n10] & 0xFF));
            }
            return n9;
        }
        catch (IOException cause) {
            throw new InternalError(cause);
        }
        catch (NoSuchAlgorithmException ex) {
            throw new SecurityException(ex.getMessage());
        }
    }
    
    private static native boolean hasStaticInitializer(Class<?> p0);
    
    private static ObjectStreamClass.FieldReflector getReflector(ObjectStreamField[] array, ObjectStreamClass objectStreamClass) throws InvalidClassException {
        Class<Object> clazz = (objectStreamClass != null && array.length > 0) ? objectStreamClass.cl : null;
        processQueue(ObjectStreamClass.Caches.reflectorsQueue, ObjectStreamClass.Caches.reflectors);
        ObjectStreamClass.FieldReflectorKey o = new ObjectStreamClass.FieldReflectorKey(clazz, array, ObjectStreamClass.Caches.reflectorsQueue);
        Reference<Object> reference = (Reference)ObjectStreamClass.Caches.reflectors.get(o);
        Object o2 = null;
        if (reference != null) {
            o2 = reference.get();
        }
        Object o3 = null;
        if (o2 == null) {
            ObjectStreamClass.EntryFuture referent = new ObjectStreamClass.EntryFuture();
            SoftReference o4 = new SoftReference(referent);
            do {
                if (reference != null) {
                    ObjectStreamClass.Caches.reflectors.remove(o, reference);
                }
                reference = (Reference<?>)ObjectStreamClass.Caches.reflectors.putIfAbsent(o, o4);
                if (reference != null) {
                    o2 = reference.get();
                }
                else {
                    continue;
                }
            } while (reference != null && o2 == null);
            if (o2 == null) {
                o3 = referent;
            }
        }
        if ((o2 instanceof ObjectStreamClass.FieldReflector)) {
            return (ObjectStreamClass.FieldReflector)o2;
        }
        if ((o2 instanceof ObjectStreamClass.EntryFuture)) {
            o2 = ((ObjectStreamClass.EntryFuture)o2).get();
        }
        else if (o2 == null) {
            try {
                o2 = new ObjectStreamClass.FieldReflector(matchFields(array, objectStreamClass));
            }
            catch (Throwable t) {
                o2 = t;
            }
            o3.set(o2);
            ObjectStreamClass.Caches.reflectors.put(o, new SoftReference(o2));
        }
        if ((o2 instanceof ObjectStreamClass.FieldReflector)) {
            return (ObjectStreamClass.FieldReflector)o2;
        }
        if ((o2 instanceof InvalidClassException)) {
            throw (InvalidClassException)o2;
        }
        if ((o2 instanceof RuntimeException)) {
            throw (RuntimeException)o2;
        }
        if ((o2 instanceof Error)) {
            throw (Error)o2;
        }
        throw new InternalError("unexpected entry: " + o2);
    }
    
    private static ObjectStreamField[] matchFields(ObjectStreamField[] array, ObjectStreamClass objectStreamClass) throws InvalidClassException {
        ObjectStreamField[] array2 = (objectStreamClass != null) ? objectStreamClass.fields : ObjectStreamClass.NO_FIELDS;
        ObjectStreamField[] array3 = new ObjectStreamField[array.length];
        Object o;
        for (int i = 0; i < array.length; array3[i] = o, ++i) {
            ObjectStreamField objectStreamField = array[i];
            o = null;
            for (int j = 0; j < array2.length; ++j) {
                ObjectStreamField objectStreamField2 = array2[j];
                if (objectStreamField.getName().equals(objectStreamField2.getName())) {
                    if ((objectStreamField.isPrimitive() || objectStreamField2.isPrimitive()) && objectStreamField.getTypeCode() != objectStreamField2.getTypeCode()) {
                        throw new InvalidClassException(objectStreamClass.name, "incompatible types for field " + objectStreamField.getName());
                    }
                    if (objectStreamField2.getField() != null) {
                        o = new ObjectStreamField(objectStreamField2.getField(), objectStreamField2.isUnshared(), false);
                    }
                    else {
                        o = new ObjectStreamField(objectStreamField2.getName(), objectStreamField2.getSignature(), objectStreamField2.isUnshared());
                    }
                }
            }
            if (o == null) {
                o = new ObjectStreamField(objectStreamField.getName(), objectStreamField.getSignature(), false);
            }
            o.setOffset(objectStreamField.getOffset());
        }
        return array3;
    }
    
    static void processQueue(ReferenceQueue<Class<?>> referenceQueue, ConcurrentMap<? extends WeakReference<Class<?>>, ?> concurrentMap) {
        Reference<Object> poll;
        while ((poll = referenceQueue.poll()) != null) {
            concurrentMap.remove(poll);
        }
    }
    
    static long access$100(Class clazz) {
        return computeDefaultSUID(clazz);
    }
    
    static Long access$700(Class clazz) {
        return getDeclaredSUID(clazz);
    }
    
    static ObjectStreamField[] access$800(Class clazz) throws InvalidClassException {
        return getSerialFields(clazz);
    }
    
    static void access$900(ObjectStreamClass objectStreamClass) throws InvalidClassException {
        objectStreamClass.computeFieldOffsets();
    }
    
    static Constructor access$1400(Class clazz) {
        return getExternalizableConstructor(clazz);
    }
    
    static Constructor access$1500(Class clazz) {
        return getSerializableConstructor(clazz);
    }
    
    static Method access$1700(Class clazz, String s, Class[] array, Class clazz2) {
        return getPrivateMethod(clazz, s, array, clazz2);
    }
    
    static ProtectionDomain[] access$2200(ObjectStreamClass objectStreamClass, Constructor constructor, Class clazz) {
        return objectStreamClass.getProtectionDomains(constructor, clazz);
    }
    
    static Method access$2400(Class clazz, String s, Class[] array, Class clazz2) {
        return getInheritableMethod(clazz, s, array, clazz2);
    }
    
    static String access$2600(Class clazz) {
        return getClassSignature(clazz);
    }
    
    static String access$2700(Class[] array, Class clazz) {
        return getMethodSignature(array, clazz);
    }
    
    static {
        NO_FIELDS = new ObjectStreamField[0];
        serialPersistentFields = ObjectStreamClass.NO_FIELDS;
        ObjectStreamClass.disableSerialConstructorChecks = (boolean)AccessController.doPrivileged(new PrivilegedAction<Boolean>() {
            @Override
            public Boolean run() {
                return "true".equals(System.getProperty("jdk.disableSerialConstructorChecks")) ? Boolean.TRUE : Boolean.FALSE;
            }
        });
        reflFactory = (ReflectionFactory)AccessController.doPrivileged(new ReflectionFactory.GetReflectionFactoryAction());
        initNative();
    }
    
    private static class Caches
    {
        static final ConcurrentMap<ObjectStreamClass.WeakClassKey, Reference<?>> localDescs;
        static final ConcurrentMap<ObjectStreamClass.FieldReflectorKey, Reference<?>> reflectors;
        private static final ReferenceQueue<Class<?>> localDescsQueue;
        private static final ReferenceQueue<Class<?>> reflectorsQueue;
        
        static {
            localDescs = new ConcurrentHashMap();
            reflectors = new ConcurrentHashMap();
            localDescsQueue = new ReferenceQueue();
            reflectorsQueue = new ReferenceQueue();
        }
    }
    
    private static class ExceptionInfo
    {
        private final String className;
        private final String message;
        
        ExceptionInfo(String className, String message) {
            this.className = className;
            this.message = message;
        }
        
        InvalidClassException newInvalidClassException() {
            return new InvalidClassException(this.className, this.message);
        }
    }
    
    private static class EntryFuture
    {
        private static final Object unset;
        private final Thread owner;
        private Object entry;
        
        private EntryFuture() {
            this.owner = Thread.currentThread();
            this.entry = ObjectStreamClass.EntryFuture.unset;
        }
        
        synchronized boolean set(Object entry) {
            if (this.entry != ObjectStreamClass.EntryFuture.unset) {
                return false;
            }
            this.entry = entry;
            this.notifyAll();
            return true;
        }
        
        synchronized Object get() {
            boolean n = false;
            while (this.entry == ObjectStreamClass.EntryFuture.unset) {
                try {
                    this.wait();
                }
                catch (InterruptedException ex) {
                    n = true;
                }
            }
            if (!(n == 0)) {
                AccessController.doPrivileged(new PrivilegedAction<Void>() {
                    @Override
                    public Void run() {
                        Thread.currentThread().interrupt();
                        return null;
                    }
                });
            }
            return this.entry;
        }
        
        Thread getOwner() {
            return this.owner;
        }
        
        static {
            unset = new Object();
        }
    }
    
    static class ClassDataSlot
    {
        final ObjectStreamClass desc;
        final boolean hasData;
        
        ClassDataSlot(ObjectStreamClass desc, boolean hasData) {
            this.desc = desc;
            this.hasData = hasData;
        }
    }
    
    private static class MemberSignature
    {
        public final Member member;
        public final String name;
        public final String signature;
        
        public MemberSignature(Field member) {
            this.member = member;
            this.name = member.getName();
            this.signature = ObjectStreamClass.access$2600(member.getType());
        }
        
        public MemberSignature(Constructor<?> member) {
            this.member = member;
            this.name = member.getName();
            this.signature = ObjectStreamClass.access$2700(member.getParameterTypes(), Void.TYPE);
        }
        
        public MemberSignature(Method member) {
            this.member = member;
            this.name = member.getName();
            this.signature = ObjectStreamClass.access$2700(member.getParameterTypes(), member.getReturnType());
        }
    }
    
    private static class FieldReflector
    {
        private static final Unsafe unsafe;
        private final ObjectStreamField[] fields;
        private final int numPrimFields;
        private final long[] readKeys;
        private final long[] writeKeys;
        private final int[] offsets;
        private final char[] typeCodes;
        private final Class<?>[] types;
        
        FieldReflector(ObjectStreamField[] fields) {
            this.fields = fields;
            int length = fields.length;
            this.readKeys = new long[length];
            this.writeKeys = new long[length];
            this.offsets = new int[length];
            this.typeCodes = new char[length];
            ArrayList<Object> list = new ArrayList();
            HashSet<Long> set = new HashSet();
            for (int i = 0; i < length; ++i) {
                ObjectStreamField objectStreamField = fields[i];
                Field field = objectStreamField.getField();
                long l = (field != null) ? ObjectStreamClass.FieldReflector.unsafe.objectFieldOffset(field) : -1L;
                this.readKeys[i] = l;
                this.writeKeys[i] = (set.add(Long.valueOf(l)) ? l : -1L);
                this.offsets[i] = objectStreamField.getOffset();
                this.typeCodes[i] = objectStreamField.getTypeCode();
                if (!objectStreamField.isPrimitive()) {
                    list.add((field != null) ? field.getType() : null);
                }
            }
            this.types = (Class[])list.toArray(new Class[list.size()]);
            this.numPrimFields = length - this.types.length;
        }
        
        ObjectStreamField[] getFields() {
            return this.fields;
        }
        
        void getPrimFieldValues(Object o, byte[] array) {
            if (o == null) {
                throw new NullPointerException();
            }
            for (int i = 0; i < this.numPrimFields; ++i) {
                long n = this.readKeys[i];
                int n2 = this.offsets[i];
                switch (this.typeCodes[i]) {
                    case 'Z': {
                        Bits.putBoolean(array, n2, ObjectStreamClass.FieldReflector.unsafe.getBoolean(o, n));
                        break;
                    }
                    case 'B': {
                        array[n2] = ObjectStreamClass.FieldReflector.unsafe.getByte(o, n);
                        break;
                    }
                    case 'C': {
                        Bits.putChar(array, n2, ObjectStreamClass.FieldReflector.unsafe.getChar(o, n));
                        break;
                    }
                    case 'S': {
                        Bits.putShort(array, n2, ObjectStreamClass.FieldReflector.unsafe.getShort(o, n));
                        break;
                    }
                    case 'I': {
                        Bits.putInt(array, n2, ObjectStreamClass.FieldReflector.unsafe.getInt(o, n));
                        break;
                    }
                    case 'F': {
                        Bits.putFloat(array, n2, ObjectStreamClass.FieldReflector.unsafe.getFloat(o, n));
                        break;
                    }
                    case 'J': {
                        Bits.putLong(array, n2, ObjectStreamClass.FieldReflector.unsafe.getLong(o, n));
                        break;
                    }
                    case 'D': {
                        Bits.putDouble(array, n2, ObjectStreamClass.FieldReflector.unsafe.getDouble(o, n));
                        break;
                    }
                    default: {
                        throw new InternalError();
                    }
                }
            }
            return;
        }
        
        void setPrimFieldValues(Object o, byte[] array) {
            if (o == null) {
                throw new NullPointerException();
            }
            for (int i = 0; i < this.numPrimFields; ++i) {
                long n = this.writeKeys[i];
                if (n != -1L) {
                    int n2 = this.offsets[i];
                    switch (this.typeCodes[i]) {
                        case 'Z': {
                            ObjectStreamClass.FieldReflector.unsafe.putBoolean(o, n, Bits.getBoolean(array, n2));
                            break;
                        }
                        case 'B': {
                            ObjectStreamClass.FieldReflector.unsafe.putByte(o, n, array[n2]);
                            break;
                        }
                        case 'C': {
                            ObjectStreamClass.FieldReflector.unsafe.putChar(o, n, Bits.getChar(array, n2));
                            break;
                        }
                        case 'S': {
                            ObjectStreamClass.FieldReflector.unsafe.putShort(o, n, Bits.getShort(array, n2));
                            break;
                        }
                        case 'I': {
                            ObjectStreamClass.FieldReflector.unsafe.putInt(o, n, Bits.getInt(array, n2));
                            break;
                        }
                        case 'F': {
                            ObjectStreamClass.FieldReflector.unsafe.putFloat(o, n, Bits.getFloat(array, n2));
                            break;
                        }
                        case 'J': {
                            ObjectStreamClass.FieldReflector.unsafe.putLong(o, n, Bits.getLong(array, n2));
                            break;
                        }
                        case 'D': {
                            ObjectStreamClass.FieldReflector.unsafe.putDouble(o, n, Bits.getDouble(array, n2));
                            break;
                        }
                        default: {
                            throw new InternalError();
                        }
                    }
                }
            }
            return;
        }
        
        void getObjFieldValues(Object o, Object[] array) {
            if (o == null) {
                throw new NullPointerException();
            }
            int i = this.numPrimFields;
            while (i < this.fields.length) {
                switch (this.typeCodes[i]) {
                    case 'L':
                    case '[': {
                        array[this.offsets[i]] = ObjectStreamClass.FieldReflector.unsafe.getObject(o, this.readKeys[i]);
                        ++i;
                        continue;
                    }
                    default: {
                        throw new InternalError();
                    }
                }
            }
            return;
        }
        
        void setObjFieldValues(Object o, Object[] array) {
            if (o == null) {
                throw new NullPointerException();
            }
            for (int i = this.numPrimFields; i < this.fields.length; ++i) {
                long offset = this.writeKeys[i];
                if (offset != -1L) {
                    switch (this.typeCodes[i]) {
                        case 'L':
                        case '[': {
                            Object x = array[this.offsets[i]];
                            if (x != null && !this.types[i - this.numPrimFields].isInstance(x)) {
                                Field field = this.fields[i].getField();
                                throw new ClassCastException("cannot assign instance of " + x.getClass().getName() + " to field " + field.getDeclaringClass().getName() + "." + field.getName() + " of type " + field.getType().getName() + " in instance of " + o.getClass().getName());
                            }
                            ObjectStreamClass.FieldReflector.unsafe.putObject(o, offset, x);
                            break;
                        }
                        default: {
                            throw new InternalError();
                        }
                    }
                }
            }
            return;
        }
        
        static {
            unsafe = Unsafe.getUnsafe();
        }
    }
    
    private static class FieldReflectorKey extends WeakReference<Class<?>>
    {
        private final String[] sigs;
        private final int hash;
        private final boolean nullClass;
        
        FieldReflectorKey(Class<?> referent, ObjectStreamField[] array, ReferenceQueue<Class<?>> q) {
            super(referent, q);
            this.nullClass = (referent == null);
            this.sigs = new String[2 * array.length];
            int i = 0;
            int n = 0;
            while (i < array.length) {
                ObjectStreamField objectStreamField = array[i];
                this.sigs[n++] = objectStreamField.getName();
                this.sigs[n++] = objectStreamField.getSignature();
                ++i;
            }
            this.hash = System.identityHashCode(referent) + Arrays.hashCode(this.sigs);
        }
        
        @Override
        public int hashCode() {
            return this.hash;
        }
        
        @Override
        public boolean equals(Object o) {
            if (o == this) {
                return true;
            }
            if ((o instanceof ObjectStreamClass.FieldReflectorKey)) {
                ObjectStreamClass.FieldReflectorKey fieldReflectorKey = (ObjectStreamClass.FieldReflectorKey)o;
                Class<Object> clazz;
                if ((!this.nullClass) ? ((clazz = (Class)this.get()) == null || clazz != fieldReflectorKey.get()) : (!fieldReflectorKey.nullClass)) {
                    return false;
                }
                if (Arrays.equals(this.sigs, fieldReflectorKey.sigs)) {
                    return true;
                }
                return false;
            }
            else {
                return false;
            }
        }
    }
    
    static class WeakClassKey extends WeakReference<Class<?>>
    {
        private final int hash;
        
        WeakClassKey(Class<?> referent, ReferenceQueue<Class<?>> q) {
            super(referent, q);
            this.hash = System.identityHashCode(referent);
        }
        
        @Override
        public int hashCode() {
            return this.hash;
        }
        
        @Override
        public boolean equals(Object o) {
            if (o == this) {
                return true;
            }
            if ((o instanceof ObjectStreamClass.WeakClassKey)) {
                Object value = this.get();
                return value != null && value == ((ObjectStreamClass.WeakClassKey)o).get();
            }
            return false;
        }
    }
}
